#!/usr/bin/python

import socket
import argparse
import os
import subprocess
import sys
import re

def gnu_dev_major(d):
    return ((d >> 8) & 0xfff) | ((d >> 32) & ~0xfff)

def gnu_dev_minor(d):
    return (d & 0xff) | ((d >> 12) & ~0xff)

def find_ploop_dev(image):
    i = re.sub('.\{.*\}$', '', image)
    out = subprocess.check_output(["/usr/sbin/ploop", "list"])
    for l in out.split("\n"):
        if i in l:
            p = l.strip().split(' ')[0]
            return "/dev/{}".format(p)

    raise Exception("Can not find image {} ploop device".format(i))

parser = argparse.ArgumentParser()
parser.add_argument("address", help="address to connect to")
parser.add_argument("port", help="port to connect to", type=int)
parser.add_argument("id", help="container id")
parser.add_argument("-d", "--disk", nargs=4, action="append", help="disk path, uuid, mount, type")
parser.add_argument("--live", action='store_true')
args = parser.parse_args()

address = (args.address, args.port)
rpc = socket.socket()
rpc.connect(address)
mem = socket.socket()
mem.connect(address)

fdsarg = []
fds = []
devs = []
shared_images = []

print subprocess.check_output(["vzctl", "--skiplock", "--skipowner", "--ignore-ha-cluster",
                               "suspend", args.id, "--create-devmap"])

if args.disk:
    for image, uuid, mount, disktype in args.disk:
        if disktype == 'local':
            fd = socket.socket()
            fd.connect(address)
            fdsarg.append('{}:{}'.format(image, fd.fileno()))
            fds.append(fd)
        elif disktype == 'shared':
            shared_images.append(os.path.dirname(image))
        else:
            raise Exception("Unknown disk {} type {}".format(image, disktype))

        if mount == '/':
            continue

        # find_ploop_dev is limited to find device historically (like /dev/ploop21617)
        # but phaul is interested in partition (like /dev/ploop21617p1). Let's hack it.
        # Fortunately st_rdev for these two differ by 1.
        d = os.stat(find_ploop_dev(image)).st_rdev + 1
        devs.append('{}:{}:{}'.format(uuid, gnu_dev_major(d), gnu_dev_minor(d)))


hargs = ["phaul", "vz", args.id,
         "--fdrpc", str(rpc.fileno()),
         "--fdmem", str(mem.fileno()),
         "--img-path", "/vz/dump/{}".format(args.id),
         "--log-file", "/var/log/phaul.log"]

if args.live:
    hargs.extend(["--pre-dump", "--auto-dedup"])
else:
    hargs.append("--no-pre-dump")

if fdsarg:
    hargs.extend(["--fdfs", ','.join(fdsarg)])
if devs:
    hargs.extend(["--vz-secondary-disks", ','.join(devs)])
if shared_images:
    hargs.extend(["--vz-shared-disks", ','.join(shared_images)])

print ' '.join(hargs)
sys.stdout.flush()
os.execv("/usr/libexec/phaul/p.haul", hargs)
